@@ -1,4 +1,4 @@
-# Copyright 2011 Yelp
+# Copyright 2011-2012 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,10 +36,11 @@ from __future__ import with_statement
 __author__ = 'David Marin <dave@yelp.com>'
 
 __credits__ = [
+    'Steve Johnson <sjohnson@yelp.com>',
     'Jennifer Snyder <jsnyder@yelp.com>',
 ]
 
-__version__ = '0.4.0'
+__version__ = '0.4.1'
 
 import inspect
 import optparse
@@ -171,7 +172,7 @@ def _to_list(x):
         return [x]
 
 
-def _run(query, dbconn, table, lock_mode, roll_back):
+def _run(query, dbconn, roll_back, table_to_lock=None):
     """Run a query with a single table locked. If an exception
     is thrown, we roll back the transaction and then unlock the table
     before re-raising the exception.
@@ -179,7 +180,8 @@ def _run(query, dbconn, table, lock_mode, roll_back):
     :param query: a function which takes a db cursor as its only argument
     :param dbconn: any DBI-compliant MySQL connection object
     :param table: table to lock while running the query
-    :param bool lock_mode: mode to lock *table* in (e.g. ``'WRITE'``)
+    :param str lock_mode: mode to lock *table* in (e.g. ``'WRITE'``), or
+                          ``None`` if *table* shouldn't be locked
     :param bool roll_back: if true, always roll back after issuing the query
 
     If there is already a transaction in progress on *dbconn*, we'll roll
@@ -193,7 +195,8 @@ def _run(query, dbconn, table, lock_mode, roll_back):
         cursor.execute('UNLOCK TABLES')
 
         cursor.execute('SET autocommit = 0')
-        cursor.execute('LOCK TABLES `%s` %s' % (table, lock_mode))
+        if table_to_lock:
+            cursor.execute('LOCK TABLES `%s` WRITE' % table_to_lock)
 
         result = query(cursor)
 
@@ -338,7 +341,7 @@ def add(dbconn, table, id_or_ids, updated=False, test=False):
     def query(cursor):
         return _add(cursor, table, ids, updated=updated)
 
-    return _run(query, dbconn, table, lock_mode='WRITE', roll_back=test)
+    return _run(query, dbconn, roll_back=test, table_to_lock=table)
 
 
 def _add(cursor, table, ids, updated=False):
@@ -398,7 +401,7 @@ def remove(dbconn, table, id_or_ids, test=False):
         _execute(cursor, sql, ids)
         return cursor.rowcount
 
-    return _run(query, dbconn, table, lock_mode='WRITE', roll_back=test)
+    return _run(query, dbconn, roll_back=test, table_to_lock=table)
 
 
 ### Getting and updating IDs ###
@@ -519,7 +522,7 @@ def get(dbconn, table, limit, lock_for=ONE_HOUR, min_loop_time=ONE_HOUR,
 
         return ids
 
-    return _run(query, dbconn, table, lock_mode='WRITE', roll_back=test)
+    return _run(query, dbconn, roll_back=test, table_to_lock=table)
 
 
 def did(dbconn, table, id_or_ids, auto_add=True, test=False):
@@ -568,7 +571,7 @@ def did(dbconn, table, id_or_ids, auto_add=True, test=False):
         _execute(cursor, sql, ids)
         return cursor.rowcount
 
-    return _run(query, dbconn, table, lock_mode='WRITE', roll_back=test)
+    return _run(query, dbconn, roll_back=test, table_to_lock=table)
 
 
 def unlock(dbconn, table, id_or_ids, auto_add=True, test=False):
@@ -632,7 +635,7 @@ def unlock(dbconn, table, id_or_ids, auto_add=True, test=False):
 
         return rowcount
 
-    return _run(query, dbconn, table, lock_mode='WRITE', roll_back=test)
+    return _run(query, dbconn, roll_back=test, table_to_lock=table)
 
 
 ### Prioritization ###
@@ -702,7 +705,7 @@ def bump(dbconn, table, id_or_ids, lock_for=0, auto_add=True, test=False):
         _execute(cursor, sql, [lock_for, lock_for] + ids)
         return cursor.rowcount
 
-    return _run(query, dbconn, table, lock_mode='WRITE', roll_back=test)
+    return _run(query, dbconn, roll_back=test, table_to_lock=table)
 
 
 ### Auditing ###
@@ -718,9 +721,10 @@ def check(dbconn, table, id_or_ids):
     locked_for)``, that is, the current time minus ``last_updated``, and
     ``lock_for`` minus the current time (both of these in seconds).
 
-    This function does not require write access to your database.
+    This function does not require write access to your database and does not
+    lock tables.
 
-    Runs this query with a read lock on *table*:
+    Runs this query:
 
     .. code-block:: sql
 
@@ -747,7 +751,7 @@ def check(dbconn, table, id_or_ids):
         return dict((id_, (since_updated, locked_for))
                     for id_, since_updated, locked_for in cursor.fetchall())
 
-    return _run(query, dbconn, table, lock_mode='READ', roll_back=True)
+    return _run(query, dbconn, roll_back=True, table_to_lock=None)
 
 
 def stats(dbconn, table):
@@ -775,11 +779,12 @@ def stats(dbconn, table):
     Only *min_id* and *max_id* can be ``None`` (when the table is empty).
     Everything else defaults to zero.
 
-    This function does not require write access to your database.
+    This function does not require write access to your database and does not
+    lock tables.
 
     :py:func:`stats` only scans locked/bumped rows and use indexes for
     everything else, so it should be very fast except in pathological cases.
-    It runs these queries with a read lock on *table*:
+    It runs these queries in a single transaction:
 
     .. code-block:: sql
 
@@ -870,7 +875,7 @@ def stats(dbconn, table):
 
         return r
 
-    return _run(query, dbconn, table, lock_mode='READ', roll_back=True)
+    return _run(query, dbconn, roll_back=True, table_to_lock=None)
 
 
 ### Object-Oriented version ###
